package lab12;

import java.util.ArrayList;

/*************************************************************************
 *  Compilation:  javac Graph.java        
 *  Execution:    java Graph input.txt
 *
 *  A graph, implemented using an array of sets.
 *  Parallel edges and self-loops allowed.
 *
 *  % java Graph tinyG.txt
 *  13 vertices, 13 edges 
 *  0: 6 2 1 5 
 *  1: 0 
 *  2: 0 
 *  3: 5 4 
 *  4: 5 6 3 
 *  5: 3 4 0 
 *  6: 0 4 
 *  7: 8 
 *  8: 7 
 *  9: 11 10 12 
 *  10: 9 
 *  11: 9 12 
 *  12: 11 9 
 *
 *  % java Graph mediumG.txt
 *  250 vertices, 1273 edges 
 *  0: 225 222 211 209 204 202 191 176 163 160 149 114 97 80 68 59 58 49 44 24 15 
 *  1: 220 203 200 194 189 164 150 130 107 72 
 *  2: 141 110 108 86 79 51 42 18 14 
 *  ...
 *  
 *  @author Andreas Papadopoulos
 *************************************************************************/

/**
 * The Graph class represents an undirected graph of vertices named 0 through
 * V-1. It supports the following operations: add an edge to the graph, iterate
 * over all of the neighbors adjacent to a vertex. Parallel edges and self-loops
 * are permitted.
 */
public class Graph {
	private final int V;
	private int E;
	private double weights[][];

	/**
	 * Create an empty graph with V vertices.
	 * 
	 * @throws java.lang.IllegalArgumentException
	 *             if V < 0
	 */
	public Graph(int V) {
		if (V <= 0)
			throw new IllegalArgumentException("Number of vertices must be > 1");
		this.V = V;
		this.E = 0;
		weights = new double[V][V];
	}

	/**
	 * Return the number of vertices in the graph.
	 */
	public int V() {
		return V;
	}

	/**
	 * Return the number of edges in the graph.
	 */
	public int E() {
		return E;
	}

	/**
	 * Add the undirected edge v-w to graph.
	 * 
	 * @throws java.lang.IndexOutOfBoundsException
	 *             unless both 0 <= v < V and 0 <= w < V
	 */
	public void addDirectedEdge(int v, int w, double weight) {
		if (weight != 0) {
			if (v < 0 || v >= V)
				throw new IndexOutOfBoundsException();
			if (w < 0 || w >= V)
				throw new IndexOutOfBoundsException();
			if (weights[v][w] == 0)
				E++;
			weights[v][w] = weight;
		}
	}

	/**
	 * Add the undirected edge v-w to graph.
	 * 
	 * @throws java.lang.IndexOutOfBoundsException
	 *             unless both 0 <= v < V and 0 <= w < V
	 */
	public void addBidirectionalEdge(int v, int w, double weight) {
		addDirectedEdge(v, w, weight);
		addDirectedEdge(w, v, weight);
	}

	/**
	 * Add the undirected edge v-w to graph.
	 * 
	 * @throws java.lang.IndexOutOfBoundsException
	 *             unless both 0 <= v < V and 0 <= w < V
	 */
	public void removeEdge(int v, int w) {
		if (v < 0 || v >= V)
			throw new IndexOutOfBoundsException();
		if (w < 0 || w >= V)
			throw new IndexOutOfBoundsException();
		if (weights[v][w] != 0)
			E--;
		weights[v][w] = 0;
	}

	public String toString() {
		String result = "";
		for (int i = 0; i < V; i++)
			for (int j = 0; j < V; j++)
				if (weights[i][j] != 0)
					result += ((char) (i + 65)) + "-" + ((char) (j + 65)) + ":"
							+ weights[i][j] + "\n";
		return result;
	}

	private class PrimEdge {
		int v1;
		int v2;
		double weight;

		PrimEdge(int v1, int v2, double weight) {
			this.v1 = v1;
			this.v2 = v2;
			this.weight = weight;
		}
		
		public String toString(){
			return ((char) (v1 + 65)) + "-" + ((char) (v2 + 65)) + ":" + weight;
		}
	}

	public void prim() {		

		// Κορυφές που προστέθηκαν στο δένδρο (Αρχικά όλα “0”)
		boolean visited[] = new boolean[V];
		for(int i=0; i<V; i++)
			visited[i] = false;
		
		// «Πιο Κοντινός Γείτονας» για κάθε i: Αρχικά κανένας
		int closest[] = new int[this.V];
		for(int i=0; i<V; i++)
			closest[i] = -1;
		
		// Απόσταση από «Κοντιν. Γείτονα» για κάθε i: Αρχικά άπειρο
		double distance[] = new double[this.V];
		for(int i=0; i<V; i++)
			distance[i] = Double.MAX_VALUE;

		// Το ΕΓΔ που θέλουμε να κτίσουμε (περιέχει ακμές (v1,v2,weight))
		ArrayList<PrimEdge> tree = new ArrayList<PrimEdge>();

		// Eπιλογή v=0 σαν αρχική κορυφή
		int v = 0;
		visited[v] = true; // Τώρα το v ανήκει στο δένδρο

		/*** ??? ***/
		
		for(int i=0; i<tree.size(); i++)
			System.out.println(tree.get(i));
	}

	private int minVertex(boolean visited[], double distance[]) {
		int min = 0;
		for (int i = 0; i < V; i++) {
			if (visited[i] == true)
				continue; // skip nodes already in MST
			if (distance[i] < distance[min])
				min = i;
		}
		return min; // Return the minimum among all distances
	}

	/**
	 * Test client.
	 */
	public static void main(String[] args) {
		Graph g = new Graph(6);
		g.addBidirectionalEdge(0, 1, 7);
		g.addBidirectionalEdge(0, 3, 9);
		g.addBidirectionalEdge(1, 2, 5);
		g.addBidirectionalEdge(1, 4, 1);
		g.addBidirectionalEdge(1, 5, 2);
		g.addBidirectionalEdge(2, 5, 6);
		g.addBidirectionalEdge(3, 4, 1);
		g.addBidirectionalEdge(4, 5, 2);

		//System.out.println(g);
		
		g.prim();

	}

}
